package org.apache.xindice.server;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: UserManager.java,v 1.1.1.1 2001/12/06 19:33:55 bradford Exp $
 */

import java.util.*;

/**
 * UserManager is an interface that is implemented to handle the
 * management of Users. StdUserManager is the standard
 * implementation of this interface.
 *
 * @see org.apache.xindice.server.standard.StdUserManager
 */

public interface UserManager {
   /**
    * getUser retrieves a User by username.
    *
    * @param username the User name
    * @return the retrieved User
    */
   User getUser(String username);

   /**
    * newUser creates a new User.  setUsername()
    * must be called before storing this new User.
    *
    * @return a new User instance
    */
   User newUser();

   /**
    * setUser stores the User to the UserManager's
    * persistent store.
    *
    * @param user the User
    */
   void setUser(User user);

   /**
    * deleteUser deletes a User from the
    * UserManager's persistent store.
    *
    * @param username the User's name
    */
   void deleteUser(String username);

   /**
    * listUsers retrieves a list of User names.  This method
    * uses startsWith to narrow the retrieval set.
    *
    * @param startsWith a filter that evaluates to startsWith*
    * @return the list of Users that match the filter
    */
   String[] listUsers(String startsWith);

   /**
    * listUsers retrieves a list of User names.  This method
    * returns all available User names.
    *
    * @return the list of all Users
    */
   String[] listUsers();
}
