package org.apache.xindice.server.components;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: BasicAuthentication.java,v 1.1.1.1 2001/12/06 19:33:55 bradford Exp $
 */

import org.apache.xindice.server.*;
import org.apache.xindice.util.*;
import java.util.*;
import sun.misc.*;

/**
 * BasicAuthentication is an implementation of ScriptFilter
 * that implements HTTP BASIC authentication.
 */

public class BasicAuthentication extends SimpleScriptComponent implements ScriptFilter {
   private static final String ALIAS = "alias";
   private static final String NAME = "name";
   private static final String REALM = "realm";

   protected Map base64cache = Collections.synchronizedMap(new HashMap()); // AuthInfo
   protected Map aliases = new HashMap(); // String

   public BasicAuthentication() {
   }

   public void setConfig(Configuration config) throws XindiceException {
      super.setConfig(config);
      config.processChildren(ALIAS,
         new ConfigurationCallback() {
            public void process(Configuration cfg) {
               aliases.put(cfg.getAttribute(NAME), cfg.getAttribute(REALM));
            }
      });
   }

   public boolean isPackageFiltered(String name) {
      return true;
   }

   protected void sendChallenge(Gateway gateway, String realm) {
      gateway.setResponseHeader("WWW-Authenticate", "BASIC realm=\""+realm+"\"");
      gateway.sendError(401, "Authorization Required");
   }

   public boolean run(Script script, Gateway gateway) {
      // Only Aliased Paths Can Be Authenticated
      String alias = gateway.getPathAlias();
      if ( alias.length() == 0 )
         return true;
      // Check If This Component Handles This Realm
      String realm = (String)aliases.get(alias);
      if ( realm == null )
         return true;
      String auth = gateway.getRequestHeader("Authorization");
      authenticate: {
         // Have They Even Provided The Authorization Header?
         if ( auth.length() == 0 )
            break authenticate;
         StringTokenizer st = new StringTokenizer(auth);
         // Does The Header Have A Type And Base64 Value?
         if ( st.countTokens() != 2 )
            break authenticate;
         String comp = st.nextToken().toUpperCase();
         // Does The Authorization Type Match What We're Looking For?
         if ( !comp.equals("BASIC") )
            break authenticate;
         try {
            auth = st.nextToken();
            AuthInfo info = (AuthInfo)base64cache.get(auth);
            if ( info == null ) {
               BASE64Decoder base64 = new BASE64Decoder();
               st = new StringTokenizer(new String(base64.decodeBuffer(auth)), ":");
               if ( st.countTokens() != 2 )
                  break authenticate;
               info = new AuthInfo(st.nextToken(), st.nextToken());
               base64cache.put(auth, info);
            }
            // Are There A User Name And Password?
            gateway.setRequestHeader("REMOTE_IDENT", info.name);
            // Get The User And Check If The Name, Password, And Realm Are Valid
            User user = kernel.getUser(info.name);
            // Have To Check Against The Script's Authentication Requirements
            int access = script.getAccessMethods();
            if ( user != null && user.getActive() && user.getPassword().equals(info.pass) && (user.getRealm(realm)&access) == access ) {
               gateway.setRequestHeader("REMOTE_USER", info.name);
               gateway.setRequestHeader("AUTH_TYPE", "BASIC");
               return true;
            }
         }
         catch ( Exception e ) {
         }
      }
      sendChallenge(gateway, realm);
      return false;
   }

   /**
    * AuthInfo
    */

   private class AuthInfo {
      public String name;
      public String pass;

      public AuthInfo(String name, String pass) {
         this.name = name;
         this.pass = pass;
      }
   }
}

