package org.apache.xindice.server.components;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: DBQueryResult.java,v 1.1.1.1 2001/12/06 19:33:56 bradford Exp $
 */

import org.apache.xindice.server.*;
import java.sql.*;

/**
 * DBQueryResult
 */

public final class DBQueryResult {
   private String alias = null;
   private DBConduit conduit = null;
   private Connection connection = null;
   private Statement statement = null;
   private ResultSet resultset = null;
   private int rowsAffected = -1;
   private boolean supportsMulti = false;

   protected DBQueryResult(DBConduit conduit, String alias, Connection connection, Statement statement, ResultSet resultset, int rowsAffected) {
      this.conduit = conduit;
      this.alias = alias;
      this.connection = connection;
      this.statement = statement;
      this.resultset = resultset;
      this.rowsAffected = rowsAffected;

      try {
         supportsMulti = connection.getMetaData().supportsMultipleResultSets();
      }
      catch ( Exception e ) {
      }
   }

   public ResultSet getResultSet() {
      return resultset;
   }

   public int getRowsAffected() {
      return rowsAffected;
   }

   public boolean hasResultSet() {
      return resultset != null;
   }

   public boolean hasRowsAffected() {
      return rowsAffected != -1;
   }

   public boolean hasMoreResults() {
      boolean done = true;
      if ( statement != null && supportsMulti ) {
         try {
            boolean results = statement.getMoreResults();
            rowsAffected = statement.getUpdateCount();
            done = ((!results) && (rowsAffected == -1));
            if ( !done ) {
               if ( results )
                  resultset = statement.getResultSet();
               else
                  rowsAffected = statement.getUpdateCount();
            }
         }
         catch ( Exception e ) {
         }
      }
      if ( statement != null && done )
         close();
      return !done;
   }

   public boolean isClosed() {
      return resultset == null && rowsAffected == -1;
   }

   private void close(boolean remove) {
      try {
         if ( statement != null )
            statement.close();
         if ( resultset != null )
            resultset.close();
      }
      catch ( Exception e ) {
         remove = true;
      }
      if ( conduit != null ) {
         if ( remove )
            conduit.removeConnection(alias, connection);
         else
            conduit.putConnection(alias, connection);
      }
      conduit = null;
      connection = null;
      statement = null;
      resultset = null;
      rowsAffected = -1;
   }

   public void close() {
      close(false);
   }

   public void remove() {
      close(true);
   }

   protected void finalize() throws Throwable {
      close(false);
   }
}
