package org.apache.xindice.server.standard;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: StdResourceManager.java,v 1.1.1.1 2001/12/06 19:33:56 bradford Exp $
 */

import org.apache.xindice.server.*;
import org.apache.xindice.util.*;
import java.util.*;

/**
 * StdResourceManager
 */

public final class StdResourceManager implements Configurable, ResourceManager, KernelAccess, Disposable, Task {
   private static final VersionProxy NullProxy = new VersionProxy("");

   private Kernel kernel;
   private Configuration config;
   private Map versions = Collections.synchronizedMap(new HashMap()); // of HashMap
   private int vultureclock = 20;

   public void setKernel(Kernel kernel) {
      this.kernel = kernel;
   }

   public void setConfig(Configuration config) {
      this.config = config;
      vultureclock = config.getIntAttribute("vulture");
      kernel.addTask(this, 30000);
   }
   
   public Configuration getConfig() {
      return config;
   }
   
   /**
    * The runTask method implements StdResourceManager's
    * resource vulture
    */
   public void runTask() {
      Map version;
      Iterator enum2;
      StdResourceItem item;
      Iterator enum = versions.values().iterator();
      while ( enum.hasNext() ) {
         version = (Map)enum.next();
         enum2 = version.values().iterator();
         while ( enum2.hasNext() ) {
            //Thread.yield();
            item = (StdResourceItem)enum2.next();
            if ( item.locked > 0 || (item.object instanceof Persistent) )
               continue;
            if ( System.currentTimeMillis() > (item.timestamp+(vultureclock * 60000)) )
               version.remove(item.name);
         }
      }
   }

   public void setResource(Versioning versioning, String name, Object object) throws ParameterException {
      String versionname = versioning.getPackageName();
      Map version = (Map)versions.get(versionname);
      if ( version == null )
         versions.put(versionname, version = new HashMap());
      StdResourceItem item = (StdResourceItem)version.get(name);
      if ( item == null )
         version.put(name, item = new StdResourceItem(name, object));
      else if ( !(item.object instanceof Persistent) ) {
         item.object = object;
         item.timestamp = System.currentTimeMillis();
      }
      else
         throw new ParameterException(name, object.toString(), "Cannot overwrite a Persistent resource");
   }

   public void setResource(String name, Object object) throws ParameterException {
      setResource(NullProxy, name, object);
   }

   public void removeResource(Versioning versioning, String name) {
      String versionname = versioning.getPackageName();
      Map version = (Map)versions.get(versionname);
      if ( version != null ) {
         StdResourceItem item = (StdResourceItem)version.get(name);
         if ( item != null && item.locked == 0 && !(item.object instanceof Persistent) )
            version.remove(name);
      }
   }

   public void removeResource(String name) {
      removeResource(NullProxy, name);
   }

   public Object getResource(Versioning versioning, String name) {
      String versionname = versioning.getPackageName();
      Map version = (Map)versions.get(versionname);
      if ( version != null ) {
         StdResourceItem item = (StdResourceItem)version.get(name);
         if ( item != null ) {
            item.timestamp = System.currentTimeMillis();
            return item.object;
         }
      }
      return null;
   }

   public Object getResource(String name) {
      return getResource(NullProxy, name);
   }

   public void lockResource(Versioning versioning, String name) {
      String versionname = versioning.getPackageName();
      Map version = (Map)versions.get(versionname);
      if ( version != null ) {
         StdResourceItem item = (StdResourceItem)version.get(name);
         if ( item != null )
            item.locked++;
      }
   }

   public void lockResource(String name) {
      lockResource(NullProxy, name);
   }

   public void unlockResource(Versioning versioning, String name) {
      String versionname = versioning.getPackageName();
      Map version = (Map)versions.get(versionname);
      if ( version != null ) {
         StdResourceItem item = (StdResourceItem)version.get(name);
         if ( item != null && item.locked > 0 )
            item.locked--;
      }
   }

   public void unlockResource(String name) {
      unlockResource(NullProxy, name);
   }

   public void dispose() {
      kernel.removeTask(this);
   }

   /**
    * StdResourceItem
    */

   private class StdResourceItem {
      public String name;
      public Object object;
      public int locked = 0;
      public long timestamp = System.currentTimeMillis();

      public StdResourceItem(String name, Object object) {
         this.name = name;
         this.object = object;
      }
   }
}
