package org.apache.xindice.server.standard;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: StdUserManager.java,v 1.1.1.1 2001/12/06 19:33:56 bradford Exp $
 */

import org.apache.xindice.server.*;
import org.apache.xindice.util.*;
import java.io.*;
import java.util.*;

/**
 * StdUserManager
 */

public final class StdUserManager implements Configurable, UserManager, KernelAccess {
   private static final String USERDIR = "userdir";
   private static final String DEFAULTS = "defaults";
   private static final String PROPERTY = "property";
   private static final String NAME = "name";
   private static final String VALUE = "value";
   
   private static final String EXT = ".user";
   
   private Kernel kernel;
   private Configuration config;
   private VersionProxy usersProxy = new VersionProxy();
   private File rootDir;
   private Map defaults = new HashMap(); // String

   public void setKernel(Kernel kernel) {
      this.kernel = kernel;
   }

   public void setConfig(Configuration config) {
      this.config = config;
      rootDir = new File(config.getAttribute(USERDIR));
      Configuration defaultsConfig = config.getChild(DEFAULTS);
      defaultsConfig.processChildren(PROPERTY,
         new ConfigurationCallback() {
            public void process(Configuration cfg) {
               defaults.put(cfg.getAttribute(NAME), cfg.getAttribute(VALUE));
            }
      });
   }
   
   public Configuration getConfig() {
      return config;
   }
   
   public User getUser(String username) {
      User user = (User)kernel.getResource(usersProxy, username);
      if ( user == null ) {
         try {
            FileInputStream fis = new FileInputStream(new File(rootDir, username+EXT));
            byte[] data = new byte[fis.available()];
            fis.read(data);
            fis.close();
            ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(data));
            user = (User)ois.readObject();
            user.setUserManager(this);
            kernel.setResource(usersProxy, username, user);
         }
         catch ( Exception e ) {
            user = null;
         }
      }
      return user;
   }

   public User newUser() {
      User user = new User("", "", defaults);
      user.setUserManager(this);
      return user;
   }

   public void setUser(User user) {
      try {
         ByteBuffer bos = new ByteBuffer(1024);
         ObjectOutputStream oos = new ObjectOutputStream(bos);
         oos.writeObject(user);
         FileOutputStream fos = new FileOutputStream(new File(rootDir, user.getUsername()+EXT));
         bos.writeTo(fos);
         fos.close();
         kernel.setResource(usersProxy, user.getUsername(), user);
      }
      catch ( Exception e ) {
      }
   }

   public void deleteUser(String username) {
      try {
         File file = new File(rootDir, username+EXT);
         file.delete();
         kernel.removeResource(usersProxy, username);
      }
      catch ( Exception e ) {
      }
   }

   public String[] listUsers(final String startsWith) {
      String[] list = rootDir.list(new FilenameFilter() {
         public boolean accept(File dir, String name) {
            if ( startsWith != null )
               return (name.startsWith(startsWith) && name.endsWith(EXT));
            else
               return (name.endsWith(EXT));
         }
      });
      int len = list.length;
      for ( int i = 0; list != null && i < len; i++ )
         list[i]=list[i].substring(0, list[i].length()-EXT.length());
      return list;
   }

   public String[] listUsers() {
      return listUsers(null);
   }
}

